/*
 * Tencent is pleased to support the open source community by making Angel available.
 *
 * Copyright (C) 2017-2018 THL A29 Limited, a Tencent company. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in 
 * compliance with the License. You may obtain a copy of the License at
 *
 * https://opensource.org/licenses/Apache-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 *
 */


package com.tencent.angel.ml.matrix.psf.get.getrows

import java.util
import java.util.{HashMap, List, Map}

import com.tencent.angel.ml.math2.vector.Vector
import com.tencent.angel.ml.matrix.psf.get.base._
import com.tencent.angel.ps.storage.vector.ServerRow
import com.tencent.angel.psagent.matrix.ResponseType
import com.tencent.angel.psagent.matrix.transport.adapter.RowSplitCombineUtils
import org.apache.commons.logging.{Log, LogFactory}

import scala.collection.JavaConversions._
import scala.collection.mutable

/**
  * Get a batch of rows function implements by the get udf.
  */
object GetRows {
  private val LOG: Log = LogFactory.getLog(classOf[GetRowsParam])
}

class GetRows(param: GetParam) extends GetFunc(param) {

  def this() = this(null)

  def partitionGet(partParam: PartitionGetParam): PartitionGetResult = {
    val rowsParam = partParam.asInstanceOf[PartitionGetRowsParam]
    val rowIndexes: List[Integer] = rowsParam.getRowIndexes

    val rows: List[ServerRow] = rowIndexes.map(psContext.getMatrixStorageManager.getRow(rowsParam.getMatrixId, _, rowsParam.getPartKey.getPartitionId))
    new PartitionGetRowsResult(rows)
  }

  def merge(partResults: List[PartitionGetResult]): GetResult = {
    val rowIndexToSplitsMap: Map[Int, List[ServerRow]] = new HashMap[Int, List[ServerRow]]
    val size = partResults.size
    for (i <- 0 until size) {
      val rowSplits: List[ServerRow] = (partResults.get(i).asInstanceOf[PartitionGetRowsResult]).getRowSplits
      rowSplits.map(row => {
        if (rowIndexToSplitsMap.containsKey(row.getRowId)) {
          rowIndexToSplitsMap.get(row.getRowId).add(row)
        } else {
          var rowList = new util.ArrayList[ServerRow]()
          rowList.add(row)
          rowIndexToSplitsMap += row.getRowId -> rowList
        }
      })
    }

    def mergedRows: mutable.Map[Integer, Vector] = rowIndexToSplitsMap map {
      case (k, v) => (k.asInstanceOf[Integer] -> RowSplitCombineUtils.combineServerRowSplits(v, param.matrixId, k))
    }


    return new GetRowsResult(ResponseType.SUCCESS, mergedRows)
  }
}
